#################################################################################################################
#
# Copyright (C) 2013-2015 X2Go Project - http://wiki.x2go.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Copyright (C) 2013-2015 Guangzhou Nianguan Electronics Technology Co.Ltd. <opensource@gznianguan.com>
# Copyright (C) 2013-2015 Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
#
#################################################################################################################
package X2Go::Telekinesis::Apps::AppCore;
use strict;
use Glib qw/TRUE FALSE/;
use Exporter qw(import);
our @EXPORT_OK = qw(getTeKiPaths  tkappDie  appDBugLog cleanUpOldAppSession checkAppSesLOCKnDirs initLOCKandDirs  getXWinPosNDim showAboutPOP getBasicServersideInfo  getCmdFlags getBasicClientsideInfo epicFailPOP);
use X2Go::Telekinesis::CommonCore qw(tcDie sanitizer genRandFID checkPID sanitizeFilePath sanitizeDirPath genRandSID clups dbugPrint);
use File::Path qw(make_path remove_tree);

sub appDBugLog {
  if (($::TEKIAPPDEBUG) or (uc($_[1]) eq "T")) {
    my $theAppName = sanitizer("tekiappname",$::APPNAME);
    if ($theAppName) {
      my $lTime = time();
      my $logLevel = 0;
      if (uc($_[1]) eq "T") {$logLevel = 1;}
      elsif (uc($_[1]) eq "W") {$logLevel = 2;}
      elsif (uc($_[1]) eq "I") {$logLevel = 9;}
      if (-d $::X2GoTKSHome) {
        open(LOG,">>$::X2GoTKSHome/appdbug_$theAppName.log") or die("$::X2GoTKSHome/appdbug_$theAppName.log");
        foreach my $logLine (split(/\n/,$_[0])) {
          if (length(clups($logLine)) > 0) {
            print LOG "$lTime|$logLevel:$logLine\n";
          }
        }
        close(LOG);
      }
    }
  }
}

sub tkappDie {
  my $error = "@_";
  appDBugLog($error,"T");
  die($error);#DIE DIE DIE!!!!!!!!!!!
}


sub getTeKiPaths {
  my %Paths;
  my $servorcli;
  if (uc($_[0]) eq "S") {$servorcli = "server";} elsif (uc($_[0]) eq "C") {$servorcli = "client";}  else {die "Someone asked for paths without specifying server or client!!";}

  if (-f "/etc/x2go/telekinesis/$servorcli/paths") {
    open(PATHS,"/etc/x2go/telekinesis/$servorcli/paths");
    while (my $pathLine = <PATHS>) {
      if (length(clups($pathLine)) > 0) {
        $pathLine =~ s/\n//g;
        if ($pathLine =~ /^\#/) {
          #NOT GOING TO DO ANYTHING FOR COMMANDS ARE WE ???  NAH...
        } elsif ($pathLine =~ /^([A-Z]{1})\:([0-9A-Z\_\-]{3,32})\=(.*)$/) {
          my $pName = $2;
          my $prePath = $3;
          if ($1 eq "D") {
            $prePath =~ s/[\s]*$//g;
            $prePath =~ s/\/$//g;
            my $cleanPath = sanitizeDirPath($prePath) or die("Sanitation of '$pName' => '$prePath' failed during loading of paths file!");
            if (($cleanPath ne 0) and (-d $cleanPath)) {
              $Paths{$pName} = $cleanPath;
            }
          }
        }
      }
    }
    close(PATHS);
  } else {die("NO PATHS FILE!");}
  return %Paths;
}

sub initLOCKandDirs {
  my $X2GoTKSHome = $_[0];
  my $AppName = sanitizer("tekiappname",$_[1]);
  my $AppSEID = sanitizer("tekiappsid",$_[2]);
  my $AppNAMEandSID = "$AppName\_$AppSEID";
  my $PID = $$;
  my $time = time();
  unless (-d "$X2GoTKSHome/local/appData") {mkdir("$X2GoTKSHome/local/appData") or epicFailPOP("Can't create the \"$X2GoTKSHome/local/appData\" dir!");}
  unless (-d "$X2GoTKSHome/remote/appData") {mkdir("$X2GoTKSHome/remote/appData") or epicFailPOP("Can't create the \"$X2GoTKSHome/remote/appData\" dir!");}
  mkdir("$X2GoTKSHome/local/appData/$AppNAMEandSID") or epicFailPOP("Can't create the \"$X2GoTKSHome/local/appData/$AppNAMEandSID\" dir!");
  mkdir("$X2GoTKSHome/remote/appData/$AppNAMEandSID") or epicFailPOP("Can't create the \"$X2GoTKSHome/remote/appData/$AppNAMEandSID\" dir!");
  open(LOCK,">$X2GoTKSHome/apps/$AppNAMEandSID.lock");print LOCK "$PID\n$time\n";close(LOCK);
}

sub checkAppSesLOCKnDirs {
  my $X2GoTKSHome = $_[0];
  my $AppName = sanitizer("tekiappname",$_[1]);
  my $AppSEID = sanitizer("tekiappsid",$_[2]);
  my $AppNAMEandSID = "$AppName\_$AppSEID";
  my $LiveSesExists = 0;
  if ($AppNAMEandSID =~ /^([a-z]{4,32})\_([a-zA-Z0-9]{3,32})$/) {
    if (-f "$X2GoTKSHome/apps/$AppNAMEandSID.lock") {	# So if there is a lock file...
      open(OLF,"$X2GoTKSHome/apps/$AppNAMEandSID.lock");my ($OldPID,$OldStartTime,undef) = <OLF>;close(OLF);# Lets have a look inside...
      $OldPID = sanitizer("num",$OldPID);$OldStartTime =~ s/\D//g;
      my ($psP,$psU,$psN,$psC) = checkPID($OldPID);	# And check if the PID is still running
      if (($psC =~ /$AppName/) and ($psU eq $::UserName)) {	# And if it is running and OURS...?!?
      ####################
      # Force NEW Stuff
        if ($::CmdFlags{'FORCENEW'}) {# About time to start cleaning up if we are ordered to FORCE a new clean session
          appDBugLog("START FORCING NEW...");
          kill("TERM",$OldPID);# Try to kill it in a friendly and polite manner....
          sleep 2;# Give it some time to die...
          my ($psP2,$psU2,$psN2,$psC2) = checkPID($OldPID);# Then Check if the pid still is running
          if (($psC2 =~ /$AppName/) and ($psU2 eq $::UserName)) {# and if it is...
            kill("KILL",$OldPID);# Try killing it in a more abrupt and brutal fassion...
            sleep 3;# Give it some time to die...
            my ($psP3,$psU3,$psN3,$psC3) = checkPID($OldPID);# Then Check again... if the pid still is running
            if (($psC3 =~ /$AppName/) and ($psU3 eq $::UserName)) {# and if it is...
              epicFailPOP("We did try to kill the damn thing ($AppName) running as pid $OldPID, but it just wont die...");
            } else {
              cleanUpOldAppSession($X2GoTKSHome,$AppNAMEandSID); # ok it dead now.... lets get on with the cleaning....
            }
          } else {
            cleanUpOldAppSession($X2GoTKSHome,$AppNAMEandSID); # ok it dead now.... lets get on with the cleaning....
          }
        } else {
          # Return some value instead of dying so we know what to do next... (load files to playlist of existing mTP etc..?)
          $LiveSesExists = 1;
        }
      # Force NEW Stuff
      ####################
      } else {# If the pid is not ours nor this app..... lets automaticaly clean up the old/dead session...
        appDBugLog("Old PID not us? :$psC:$AppName:	:$psU:$::UserName:");
        cleanUpOldAppSession($X2GoTKSHome,$AppNAMEandSID);
      }
    } else {
      if ((-d "$X2GoTKSHome/local/appData/$AppNAMEandSID/") or (-d "$X2GoTKSHome/local/appData/$AppNAMEandSID/")) {cleanUpOldAppSession($X2GoTKSHome,$AppNAMEandSID);}
    }
  }
  return $LiveSesExists;
}

sub cleanUpOldAppSession {
  my $X2GoTKSHome = $_[0];
  my $tmpAppNAMEandSID = $_[1];
  if ($tmpAppNAMEandSID =~ /^([a-z]{4,32})\_([a-zA-Z0-9]{3,32})$/) {
    my $AppName = sanitizer("tekiappname",$1);
    my $AppSEID = sanitizer("tekiappsid",$2);
    my $AppNAMEandSID = "$AppName\_$AppSEID";
    appDBugLog("Start cleaning old session junk...\nCleaning up $AppNAMEandSID...");
    if (-d "$X2GoTKSHome/local/appData/$AppNAMEandSID") {
      remove_tree("$X2GoTKSHome/local/appData/$AppNAMEandSID") or tcDie("Can't remove the \"$X2GoTKSHome/local/appData/$AppNAMEandSID\" dir!\ There is probably still some junk left in it?!");
    }
    if (-d "$X2GoTKSHome/remote/appData/$AppNAMEandSID") {
      remove_tree("$X2GoTKSHome/remote/appData/$AppNAMEandSID") or tcDie("Can't remove the \"$X2GoTKSHome/remote/appData/$AppNAMEandSID\" dir!\ There is probably still some junk left in it?!");
    }
    if (-f "$X2GoTKSHome/apps/$AppNAMEandSID.lock") {
      unlink("$X2GoTKSHome/apps/$AppNAMEandSID.lock");
    }
    appDBugLog("DONE cleaning old session junk!");
  }
}


sub getCmdFlags {
  my %theArgs;
  foreach my $ent (@_) {
    $ent =~ s/\n//g;
    if ($ent =~ /^\-set([A-Z0-9]{4,24})\=(.*)$/) {
      $theArgs{$1} = $2;
    }
  }
  return %theArgs;
}

sub getBasicServersideInfo {
  my $UserHome    = sanitizeDirPath($ENV{'HOME'});# Over zealusly initiate the variable that holds the users HOME folder path and user name
  unless (-d $UserHome) {epicFailPOP("User got no HOME directory?");}
  my $UserName = sanitizer("pnixusername",$ENV{'USER'});
  if ($UserName eq 0) {epicFailPOP("Is \"$UserName\" a valid portable UNIX username?\nNah.... Didn't think so....\nWe're done! BYE BYE!\n");}
  appDBugLog("Running as USER:	$UserName\nWith HOME Dir:		$UserHome");
  #################################################################################################################################
  # Again with the checking of the x2go and x2go session directory.
  unless ($ENV{'X2GO_SESSION'}) {epicFailPOP("NO X2GO_SESSION ENV!!!\nWe're not in a X2Go session ARE WE?!!\n");}
  my $X2GoSID = sanitizer("anumazcsdaus",$ENV{'X2GO_SESSION'});
  my $X2GoSesHome = sanitizeDirPath("$UserHome/.x2go/C-$X2GoSID");
  my $X2GoTKSHome = "$X2GoSesHome/telekinesis";
  appDBugLog("X2Go SesID ENV: $X2GoSID\nX2GoTKSHome: $X2GoTKSHome");
  my $TeKiLSockPath = "$X2GoTKSHome/local/sock";
  unless (-S $TeKiLSockPath) {
    epicFailPOP("No Telekinesis local socket available at: $TeKiLSockPath\n\nIs Telekinesis even running?");
  } else {
    # This stuff moved somewhere else...
  }
  return ($UserName,$UserHome,$X2GoSID,$X2GoSesHome,$X2GoTKSHome,$TeKiLSockPath);
}


sub getBasicClientsideInfo {
  my $inX2GoSID = $_[0];
  my $UserHome    = sanitizeDirPath($ENV{'HOME'});# Over zealusly initiate the variable that holds the users HOME folder path and user name
  unless (-d $UserHome) {epicFailPOP("User got no HOME directory?");}
  my $UserName = sanitizer("pnixusername",$ENV{'USER'});
  if ($UserName eq 0) {epicFailPOP("Is \"$UserName\" a valid portable UNIX username?\nNah.... Didn't think so....\nWe're done! BYE BYE!\n");}
  appDBugLog("Running as USER:	$UserName\nWith HOME Dir:		$UserHome");
  #################################################################################################################################
  # Again with the checking of the x2go and x2go session directory.
  unless ($inX2GoSID) {epicFailPOP("NO X2GO_SESSION ENV!!!\nWe're not in a X2Go session ARE WE?!!\n");}
  my $X2GoSID = sanitizer("anumazcsdaus",$inX2GoSID);
  my $X2GoSesHome = sanitizeDirPath("$UserHome/.x2go/S-$X2GoSID");
  my $X2GoTKCHome = "$X2GoSesHome/telekinesis";
  appDBugLog("X2Go SesID ENV: $X2GoSID\nX2GoTKCHome: $X2GoTKCHome");
  my $TeKiLSockPath = "$X2GoTKCHome/local/sock";
  unless (-S $TeKiLSockPath) {
    epicFailPOP("No Telekinesis local socket available at: $TeKiLSockPath\n\nIs Telekinesis even running?");
  } else {
    # This stuff moved somewhere else...
  }
  return ($UserName,$UserHome,$X2GoSID,$X2GoSesHome,$X2GoTKCHome,$TeKiLSockPath);
}

sub showAboutPOP {
  my $dialog = Gtk2::Dialog->new ("About $::APPDNAME v$::APPVER", undef,  [qw/modal destroy-with-parent/],
        'gtk-ok' => 'ok',
  );

  $dialog->set_resizable(0);
  $dialog->signal_connect( delete_event => sub {$dialog->destroy;} );


  my $vBox0 = Gtk2::VBox->new (FALSE, 6);
  my $hBox0 = Gtk2::HBox->new (FALSE, 6);


  my $swW = 580;
  my $swH = 400;

  if (-f "$::theSHAREDIR/aboutimg.png") {
    my $pBuf0 = Gtk2::Gdk::Pixbuf->new_from_file ("$::theSHAREDIR/aboutimg.png");
    my $aboutImage = Gtk2::Image->new_from_pixbuf($pBuf0);#$aboutImage->set_size_request(600,280);
    my $eb0 = Gtk2::EventBox->new;$eb0->set_size_request(600,280);$eb0->add($aboutImage);
    $vBox0->pack_start ($eb0, TRUE, TRUE, 0);
    $swH = 200;
  }

  if (-f "$::theSHAREDIR/aboutbylogo.png") {
    my $pBuf1 = Gtk2::Gdk::Pixbuf->new_from_file ("$::theSHAREDIR/aboutbylogo.png");
    my $aboutByImage = Gtk2::Image->new_from_pixbuf($pBuf1);#$aboutByImage->set_size_request(180,150);
    my $eb1 = Gtk2::EventBox->new;$eb1->set_size_request(190,190);$eb1->add($aboutByImage);
    $hBox0->pack_start ($eb1, TRUE, TRUE, 0);
    $swW = 400;
  }


  my $lFrame = Gtk2::Frame->new("Terms, Conditions and License");
  $lFrame->set_border_width(1);
  my $sw = Gtk2::ScrolledWindow->new (undef, undef);
  $sw->set_shadow_type ('etched-out');
  $sw->set_policy ('automatic', 'automatic');
  $sw->set_size_request ($swW, $swH);
  $sw->set_border_width(1);
  Gtk2::Rc->parse_string(<<__);
    style "CnL_text" {\n  font_name ="sans 7"\n  text[NORMAL] = "#222222"\n  base[NORMAL] = "#ffffff"\n }\n  widget "*Text*" style "CnL_text"\n
__

  my $tview = Gtk2::TextView->new();
  my @content;
  if (-f "$::theSHAREDIR/copyright.txt") {
    open(TEXT,"$::theSHAREDIR/copyright.txt");@content =  <TEXT>;close(TEXT);
  }
  push @content, "\n\n";
  if (-f "$::theSHAREDIR/license.txt") {

    open(TEXT,"$::theSHAREDIR/license.txt");push @content,  <TEXT>;close(TEXT);
  }


  my $buffer = $tview->get_buffer();
  $buffer->set_text("@content");
  $sw->add($tview);
  $lFrame->add($sw);

  $hBox0->pack_start ($lFrame, TRUE, TRUE, 0);
  $vBox0->pack_start ($hBox0, TRUE, TRUE, 0);

  $vBox0->show_all;
  $dialog->vbox->pack_start ($vBox0, TRUE, TRUE, 0);
  $dialog->set_default_response ('ok');
  if ($dialog->run eq 'ok') {$dialog->destroy;}

}

sub getXWinPosNDim {
  my $XID = $_[0];
  my @result = `xwininfo -id "$XID" -shape`;
  my %Xpos;
  foreach my $line (@result) {
    $line =~ s/[\n\ ]//g;
    if ($line =~ /^Width:(\d{1,5})$/) {
      $Xpos{'width'} = $1;
    } elsif ($line =~ /^Height:(\d{1,5})$/) {
      $Xpos{'height'} = $1;
    } elsif ($line =~ /^Absoluteupper-leftX:([\-\d]{1,5})$/) {
      $Xpos{'absupleftx'} = $1;
    } elsif ($line =~ /^Absoluteupper-leftY:([\-\d]{1,5})$/) {
      $Xpos{'absuplefty'} = $1;
    } elsif ($line =~ /^Relativeupper-leftX:([\-\d]{1,5})$/) {
      $Xpos{'relupleftx'} = $1;
    } elsif ($line =~ /^Relativeupper-leftY:([\-\d]{1,5})$/) {
      $Xpos{'reluplefty'} = $1;
    }
  }

  return ($Xpos{'relupleftx'},$Xpos{'reluplefty'},$Xpos{'absupleftx'},$Xpos{'absuplefty'},$Xpos{'height'},$Xpos{'width'});
}

sub epicFailPOP {
  unless ($ENV{'EPICFAIL'} or $::EPICFAIL) {
    my $VerboseStuff = $_[0];$VerboseStuff =~ s/^/ /g;$VerboseStuff =~ s/\n/\n /g;
    appDBugLog("$VerboseStuff");
    Gtk2->init;
    Gtk2::Rc->parse_string(<<__);
  style "err_text" {
    font_name ="sans 10"
    text[NORMAL] = "#FF0000"
    base[NORMAL] = "#000000"
    GtkTextView::cursor-color = "red"
  }

  widget "*Text*" style "err_text"
__
    my $window = Gtk2::Window->new;
    $window->set_title('Epic FAIL!');
    $window->signal_connect( delete_event => sub { exit } );
    $window->set_default_size( 500, 300 );
    $window->set_keep_above(1);
    my $vbox = Gtk2::VBox->new;$window->add($vbox);
    $vbox->pack_start( Gtk2::Label->new ("$ENV{'APPNAME'} v$ENV{'APPVER'} experienced an EPIC FAIL:"), FALSE, FALSE, 1 );
    my $scroller = Gtk2::ScrolledWindow->new;$vbox->add($scroller);
    my $textview = Gtk2::TextView->new;$textview ->set_editable (0);$textview ->can_focus(0);
    my $buffer = $textview->get_buffer;
    $scroller->add($textview);
    my $button = Gtk2::Button->new_from_stock('gtk-quit');
    $button->signal_connect( clicked => sub { exit; } );
    $vbox->pack_start( $button, FALSE, FALSE, 0 );
    $buffer->insert( $buffer->get_end_iter, "$VerboseStuff\n");
    $window->show_all;
    if ($::MainAppWindow) {
      $::MainAppWindow->destroy();
    }
  }
  $::EPICFAIL = 1;
  $ENV{'EPICFAIL'} = 1;
  Gtk2->main;
  Gtk2->main_quit;
  return 0;
}


1;
